﻿//Copyright © 2021 by ANTRAS GmbH, covered by AGPL-3.0-or-later
using ANTRAS.PdfConversion.service.helper;
using ANTRAS.PdfConversion.service.Properties;
using ANTRAS.PdfConversion.service.threads;
using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration.Install;
using System.Diagnostics;
using System.IO;
using System.ServiceProcess;
using System.Threading;

namespace ANTRAS.PdfConversion.service {
	public class Service : ServiceBase {
		private ThreadAndWorkerList _ThreadAndWorkerList;
		private IContainer components = null;

		public Service() {
			InitializeComponent();
		}

		protected override void Dispose(bool disposing) {
			if(disposing && (components != null)) {
				components.Dispose();
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent() {
			components = new System.ComponentModel.Container();
			this.ServiceName = "ANTRAS.PdfConversion.service";
		}

		protected override void OnStart(string[] args) {
			base.OnStart(args);
			_ThreadAndWorkerList = new ThreadAndWorkerList();

			bool settingsOk = true;
			if(string.IsNullOrWhiteSpace(Settings.Default.InputDirecory)) {
				settingsOk = false;
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service", "InputDirecory not configured", EventLogEntryType.Error, 0);
			}
			if(string.IsNullOrWhiteSpace(Settings.Default.OuputDirecory)) {
				settingsOk = false;
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service", "OuputDirecory not configured", EventLogEntryType.Error, 0);
			}
			if(string.IsNullOrWhiteSpace(Settings.Default.TempDirectory)) {
				settingsOk = false;
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service", "TempDirectory not configured", EventLogEntryType.Error, 0);
			}
			if(string.IsNullOrWhiteSpace(Settings.Default.PathToGhostscriptDll64)) {
				settingsOk = false;
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service", "PathToGhostscriptDll64 not configured", EventLogEntryType.Error, 0);
			}
			if(File.Exists(Settings.Default.PathToGhostscriptDll64)) {
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("PathToGhostscriptDll64 {0}, DLL is present.", Settings.Default.PathToGhostscriptDll64), EventLogEntryType.Information, 0);
			}

			if(settingsOk) {
				if(!Helper.TryToGuaranteePath(Settings.Default.InputDirecory)) {
					settingsOk = false;
					EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("InputDirecory {0} not existing.", Settings.Default.InputDirecory), EventLogEntryType.Error, 0);
				}
				if(!Helper.TryToGuaranteePath(Settings.Default.OuputDirecory)) {
					settingsOk = false;
					EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("OuputDirecory {0} not existing.", Settings.Default.OuputDirecory), EventLogEntryType.Error, 0);
				}
				if(!Helper.TryToGuaranteePath(Settings.Default.TempDirectory)) {
					settingsOk = false;
					EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("TempDirectory {0} not existing.", Settings.Default.TempDirectory), EventLogEntryType.Error, 0);
				}
				if(!File.Exists(Settings.Default.PathToGhostscriptDll64)) {
					settingsOk = false;
					EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("PathToGhostscriptDll64 {0}, DLL not existing.", Settings.Default.PathToGhostscriptDll64), EventLogEntryType.Error, 0);
				}
			}
			if(settingsOk) {
				PrepareThreads();
				StartAllThreads();
			} else {
				Stop();
			}
		}

		protected override void OnStop() {
			StopAllThreads();
			base.OnStop();
		}

		private void PrepareThreads() {
			EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("Preparing Threads"), EventLogEntryType.Information, 0);
			#region main conversion thread
			ConvertThread worker = new ConvertThread(Settings.Default.InputDirecory, Settings.Default.OuputDirecory);
			ThreadStart st = new ThreadStart(worker.Start);
			ThreadAndWorker taw = new ThreadAndWorker(worker, new Thread(st));
			_ThreadAndWorkerList.Add(taw);
			#endregion main conversion thread
		}

		private void StartAllThreads() {
			EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("Starting Threads"), EventLogEntryType.Information, 0);
			foreach(ThreadAndWorker taw in _ThreadAndWorkerList) {
				taw.Worker.ServiceIsRunning = true;
				taw.Thread.Start();
			}
		}
		private void StopAllThreads() {
			// inform threads to stop 
			foreach(ThreadAndWorker taw in _ThreadAndWorkerList) {
				taw.Worker.Stop();
			}
			Thread.Sleep(1000); // give them some time to stop

			// check is at least one thread still working 
			bool someoneIsStillWorking = false; ;
			foreach(ThreadAndWorker taw in _ThreadAndWorkerList) {
				someoneIsStillWorking = taw.Worker.IamWorking || someoneIsStillWorking;
			}
			if(someoneIsStillWorking) {
				Thread.Sleep(2000); // give them some more time to stop
			}

			// finally join the threads
			foreach(ThreadAndWorker taw in _ThreadAndWorkerList) {
				taw.Thread.Join(new TimeSpan(0, 0, 20));
			}
		}
	}

	[RunInstaller(true)]
	public class ServiceInstallerBase : Installer {
		#region private members
		private ServiceProcessInstaller _ServiceProcessInstaller;
		private ServiceInstaller _ServiceInstaller;
		private IContainer _Components = null;

		private string _ServiceDescription = "Konvertiert einzelne Seiten von PDF Dokumenten in Bilder.";
		private string _ServiceDisplayName = "ANTRAS PDF Conversion Service";
		private string _ServiceName = "ANTRAS.PdfConversion.Service";
		#endregion

		#region constructors
		public ServiceInstallerBase() {
			Init();
			InitializeComponent();
		}
		#endregion

		protected void Init() {
		} 

		#region install and uninstall
		protected override void OnBeforeInstall(IDictionary savedState) {
			SetServiceName();
			base.OnBeforeInstall(savedState);

		}
		protected override void OnBeforeUninstall(IDictionary savedState) {
			SetServiceName();
			base.OnBeforeUninstall(savedState);
		}

		protected void SetServiceName() {
			// read 'environnement' as Suffix for the Servicename. Usefull, if you like to install the service more than once
			string env;
			if(!String.IsNullOrEmpty(Context.Parameters["environnement"])) {
				env = Context.Parameters["environnement"].Trim();
				_ServiceInstaller.ServiceName = _ServiceInstaller.ServiceName + "." + env;
				_ServiceInstaller.DisplayName = _ServiceInstaller.DisplayName + " (" + env + ")";
				_ServiceInstaller.Description = "Environnement " + env.ToUpper() + ":" + _ServiceInstaller.Description;
			}
		}

		protected string ServiceDescription { get { return _ServiceDescription; } set { _ServiceDescription = value; } }
		protected string ServiceName { get { return _ServiceName; } set { _ServiceName = value; } }
		protected string ServiceDisplayName { get { return _ServiceDisplayName; } set { _ServiceDisplayName = value; } }

		#endregion

		protected override void Dispose(bool disposing) {
			if(disposing && (_Components != null)) {
				_Components.Dispose();
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent() {
			_Components = new Container();
			_ServiceProcessInstaller = new ServiceProcessInstaller();
			_ServiceInstaller = new ServiceInstaller();
			_ServiceProcessInstaller.Password = null;
			_ServiceProcessInstaller.Username = null;
			_ServiceInstaller.Description = ServiceDescription;
			_ServiceInstaller.DisplayName = ServiceDisplayName;
			_ServiceInstaller.ServiceName = ServiceName;
			Installers.AddRange(new Installer[] { _ServiceProcessInstaller, _ServiceInstaller });

		}
	}
}
