﻿using ANTRAS.PdfConversion.service.helper;
using ANTRAS.PdfConversion.service.Properties;
using Ghostscript.NET;
using Ghostscript.NET.Rasterizer;
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace ANTRAS.PdfConversion.service.conversion {
	public class Pdf2Image {
		#region local variables
		private string _PdfFileName;
		private string _TargetPath;
		private string _FileNamePattern;
		private ImageFormat _ImageFormat = ImageFormat.Png;
		private int _Dpi = 100;
		#endregion local variables

		#region constructors
		/// <summary>
		/// Convert a PDF file. The images are created in the output directory 'Settings.OuputDirecory'.
		/// </summary>
		/// <param name="pdfFilename">PDF to convert</param>
		public Pdf2Image(string pdfFilename) {
			_PdfFileName = pdfFilename;
		}
		/// <summary>
		/// Convert a PDF file. The images are created in the targetPath, which must exist.
		/// </summary>
		/// <param name="pdfFilename">PDF to convert</param>
		/// <param name="targetPath">path where the images should be stored</param>
		public Pdf2Image(string pdfFilename, string targetPath) {
			_PdfFileName = pdfFilename;
			_TargetPath = targetPath;
		}

		/// <summary>
		/// Convert a PDF file. The images are created in the targetPath, which must exist.
		/// </summary>
		/// <param name="pdfFilename">PDF to convert</param>
		/// <param name="targetPath">path where the images should be stored</param>
		/// <param name="fileNamePattern">Filenamepattern for storing converted imaqges , {0} will be used for the page number, {1} for the imageformat</param>
		public Pdf2Image(string pdfFilename, string targetPath, string fileNamePattern) {
			_PdfFileName = pdfFilename;
			_TargetPath = targetPath;
			_FileNamePattern = fileNamePattern;
		}

		/// <summary>
		/// Convert a PDF file. The images are created in the targetPath, which must exist.
		/// </summary>
		/// <param name="pdfFilename">PDF to convert</param>
		/// <param name="targetPath">path where the images should be stored</param>
		/// <param name="fileNamePattern">Filenamepattern fpr storing converted imaqges , {0} is for the original filename without the extension, {1} will be used for the page number, {2} for the imageformat</param>
		/// <param name="dpi">DPI to be used during conversion, default is 300</param>
		/// <param name="imageFormat">Imageformat of the converted page(s)</param>
		public Pdf2Image(string pdfFilename, string targetPath, string fileNamePattern, int dpi, ImageFormat imageFormat) {
			_PdfFileName = pdfFilename;
			_TargetPath = targetPath;
			_FileNamePattern = fileNamePattern;
			if(dpi > 0) {
				_Dpi = dpi;
			}
			_ImageFormat = imageFormat;
		}
		#endregion constructors

		#region conversion
		/// <summary>
		/// Converts the first page to an image
		/// </summary>
		/// <returns></returns>
		public StringList Convert() {
			IntList pages = new IntList();
			pages.Add(1);
			return Convert(pages);
		}
		public StringList Convert(int pageNo) {
			IntList pages = new IntList();
			pages.Add(pageNo);
			return Convert(pages);
		}
		public StringList ConvertAll() {
			return Convert(null);
		}

		/// <summary>
		/// Converts the page(s) to images
		/// </summary>
		/// <param name="pageNumbers">Page numbers to convert, null means all</param>
		/// <returns>List of paths to the converted images </returns>
		private StringList Convert(IntList pageNumbers) {

			StringList ret = new StringList();
			string pathDll = Settings.Default.PathToGhostscriptDll64;
			string tmpfile = Path.Combine(Settings.Default.TempDirectory, Helper.GetRandomFilename("pdf"));
			try {
				File.Copy(PdfFileName, tmpfile, true);
				try {
					GhostscriptVersionInfo version = new GhostscriptVersionInfo(new Version(0, 0, 0), pathDll, string.Empty, GhostscriptLicense.GPL);
					using(GhostscriptRasterizer rasterizer = new GhostscriptRasterizer()) {
						try {
							rasterizer.Open(tmpfile, version, false);
							if(pageNumbers == null) {
								pageNumbers = new IntList();
							}
							if(pageNumbers.Count < 1) {
								for(int i = 0; i < rasterizer.PageCount; i++) {
									pageNumbers.Add(i + 1);
								}
							}

							foreach(int pn in pageNumbers) {
								if(pn >= 0 && pn <= rasterizer.PageCount) {
									string imageFile = GetFormattedFullFileName(pn);
									if(!File.Exists(imageFile) || Helper.IsFileA_newerThan_B(PdfFileName, imageFile)) {
										try {
											Image img = rasterizer.GetPage(Dpi, pn);
											img.Save(imageFile, ImageFormat);
											ret.Add(imageFile);
											EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
												, string.Format("Pdf2Image.Convert() has written {0}", imageFile)
												, EventLogEntryType.Information
												, 0
												);

										} catch(Exception ex) {
											EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
												, string.Format("Pdf2Image.Convert() on rasterize and save, page={0} of {1}. Try to save {4}. {2}. {3}", pn, PdfFileName, ex.Message, ex.StackTrace, imageFile)
												, EventLogEntryType.Error
												, 0
												);
										}
									}
								}
							}
						} catch(Exception ex) {
							EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
								, string.Format("Pdf2Image.Convert() while open file {0} in rasterizer, GhostScript DLL in {1}. {2}. {3}", PdfFileName, pathDll, ex.Message, ex.StackTrace)
								, EventLogEntryType.Error
								, 0
								);
						} finally {
							rasterizer.Close();
						}
					}
				} catch(Exception ex) {
					EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
						, string.Format("Pdf2Image.Convert() while creation an instance of {0}. {1}. {2}", pathDll, ex.Message, ex.StackTrace)
						, EventLogEntryType.Error
						, 0
						);
				}
			} catch(Exception ex) {
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
					, string.Format("Pdf2Image.Convert() while copy PDF to tempfile {0}. {1}. {2}", pathDll, ex.Message, ex.StackTrace)
					, EventLogEntryType.Error
					, 0
					);

			} finally {
				if(File.Exists(tmpfile)) {
					File.Delete(tmpfile);
				}
			}

			return ret;
		}
		#endregion conversion

		#region properties
		public string GetFormattedFullFileName(int pageNo) { return Path.Combine(TargetPath, GetFormattedFileName(pageNo)); }
		public string GetFormattedFileName(int pageNo) { return string.Format(FileNamePattern, Path.GetFileNameWithoutExtension(PdfFileName), pageNo, ImageFormat.ToString()); }
		public string PdfFileName { get { return _PdfFileName; } }
		public string TargetPath { get { return _TargetPath != null ? _TargetPath : Settings.Default.OuputDirecory; } }

		/// <summary>
		/// default is [orig filename without extension]_[DPI]_p[PageNo].[extension from ImageFormat] => "{0}_100p{1}.{2}" 
		/// example: Origfile MyPdfFile.pdf => MyPdfFile_p1.png
		/// </summary>
		public string FileNamePattern {
			get {
				return _FileNamePattern == null ? "{0}_" + Dpi.ToString() + "_p{1}.{2}" : _FileNamePattern;
			}
			set { _FileNamePattern = value; }
		}
		public int Dpi { 
			get {
				if(_Dpi < 1) {
					_Dpi = 100;
				}
				return _Dpi;
			} 
		}
		public ImageFormat ImageFormat { get { return _ImageFormat; } }
		#endregion properties		
	}

}
