﻿//Copyright © 2021 by ANTRAS GmbH, covered by AGPL-3.0-or-later
using ANTRAS.PdfConversion.service.helper;
using ANTRAS.PdfConversion.service.Properties;
using System;
using System.Diagnostics;
using System.Drawing.Imaging;
using System.IO;
using System.Xml;

namespace ANTRAS.PdfConversion.service.conversion {

	public class Request {
		#region local variables
		private string _Request;
		private string _PathToSourceDocument = "";
		private string _StoreResultsInPath = "";
		private string _FileNamePattern;
		private int _PageNo;
		private int _Dpi = 100;
		private ImageFormat _ImageFormat = ImageFormat.Png;
		private bool _HasCommandFile;
		#endregion local variables

		#region private constructors
		public Request(string file) {
			_Request = file;
			FileInfo f = new FileInfo(file);
			if(f.Exists) {
				if(string.Compare(f.Extension, ".xml", true) == 0) {
					Parse(file);
				} else if(string.Compare(f.Extension, ".pdf", true) == 0) {
					PathToSourceDocument = file;
					StoreResultsInPath = Settings.Default.OuputDirecory;
					PageNo = 0;
					Dpi = 100;
					ImageFormat = ImageFormat.Png;
				}
			}
		}
		#endregion  private constructors

		#region Properties
		public string PathToSourceDocument { get { return _PathToSourceDocument; } set { _PathToSourceDocument = value; } }

		/// <summary>
		/// default is the path of the source document
		/// </summary>
		public string StoreResultsInPath { get { return _StoreResultsInPath; } set { _StoreResultsInPath = value; } }

		/// <summary>
		/// default is [orig filename without extension]_p[PageNo].[extension from ImageFormat] => "{0}_p{1}.{2}" 
		/// example: Origfile MyPdfFile.pdf => MyPdfFile_p1.png
		/// </summary>
		public string FileNamePattern { 
			get {
				return _FileNamePattern != null ? _FileNamePattern :  "{0}_" + Dpi.ToString() + "_p{1}.{2}";
			}
			set { _FileNamePattern = value; }
		}

		/// <summary>
		/// -1 or 0 means all pages
		/// </summary>
		public int PageNo { get { return _PageNo; } set { _PageNo = value; } }

		/// <summary>
		/// default is 100 DPI
		/// </summary>
		public int Dpi { get { return _Dpi; } set { _Dpi = value; } }

		/// <summary>
		/// default is png
		/// </summary>
		public ImageFormat ImageFormat { get { return _ImageFormat; } set { _ImageFormat = value; } }

		private bool HasCommandFile { get { return _HasCommandFile; } set { _HasCommandFile = value; } }
		#endregion  Properties


		#region XML handling
		private void Parse(string xmlFile) {
			if(File.Exists(xmlFile)) {
				try {
					XmlDocument doc = new XmlDocument();
					doc.Load(xmlFile);
					XmlElement root = doc.SelectSingleNode("PdfToImage") as XmlElement;
					if(root != null) {
						{
							string val = root.GetAttribute("PageNo").Trim();
							PageNo = Helper.IsInteger(val) ? int.Parse(val) : -1;
						}
						{
							string val = root.GetAttribute("Dpi").Trim();
							Dpi = Helper.IsInteger(val) ? int.Parse(val) : 100;
						}
						{
							string val = root.GetAttribute("ImageFormat").Trim();
							try {
								if(!string.IsNullOrWhiteSpace(val)) {
									ImageFormat = (ImageFormat)Enum.Parse(typeof(ImageFormat), val, true);
								}
							} catch(Exception) {
								EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
									, string.Format("Request.Parse cannot parse ImageFormat '{0}' and will use 'Png' instead.", val)
									, EventLogEntryType.Information
									, 0
									);
							}
						}

						PathToSourceDocument = GetStringValueFromElement(root.SelectSingleNode("PathToSourceDocument") as XmlElement);
						StoreResultsInPath = GetStringValueFromElement(root.SelectSingleNode("StoreResultsInPath") as XmlElement);

						string fnp = GetStringValueFromElement(root.SelectSingleNode("FileNamePattern") as XmlElement);
						FileNamePattern = string.IsNullOrWhiteSpace(fnp) ? null : fnp;


					} else {
						EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
							, string.Format("XML Request {0} not valid: {0}. Root node PdfToImage not existent", xmlFile)
							, EventLogEntryType.Error
							);
					}

				} catch(Exception ex) {
					EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
						, string.Format("Request.Parse not load XML: {0}. {1}. {2}", xmlFile, ex.Message, ex.StackTrace)
						, EventLogEntryType.Error
						, 0
						);
				}


			}
		}
		private string GetStringValueFromElement(XmlElement elem) {
			string ret = null;

			if(elem != null) {
				
				foreach(XmlNode n in elem.ChildNodes) {
					if(n is XmlCDataSection) {
						XmlCDataSection cdata = (XmlCDataSection)n;
						ret = cdata.Value;
						break;
					}
				}
				if(ret == null) {
					ret = elem.InnerText;
				}
			}

			return ret;
		}
		#endregion XML handling

		#region processing
		public bool Process() {
			bool ret = false;

			Pdf2Image p2i = new Pdf2Image(PathToSourceDocument, StoreResultsInPath, FileNamePattern, Dpi, ImageFormat);
			if(PageNo > 0) {
				p2i.Convert(PageNo);
			} else {
				p2i.ConvertAll();
			}

			try {
				File.Delete(_Request);
			} catch(Exception ex) {
				EventLog.WriteEntry("ANTRAS.PdfConversion.Service"
					, string.Format("Request.Process could not remove {0}. {1}. {2}", _Request, ex.Message, ex.StackTrace)
					, EventLogEntryType.Error
					, 0
					);
			}

			return ret;
		}
		#endregion

	}

}
