﻿//Copyright © 2021 by ANTRAS GmbH, covered by AGPL-3.0-or-later
using ANTRAS.PdfConversion.service.conversion;
using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Timers;

namespace ANTRAS.PdfConversion.service.threads {

	public abstract class ThreadWorkerBase {
        protected volatile bool _ServiceIsRunning = false;
        protected volatile bool _IamWorking = false;
        public ThreadWorkerBase() { }
        public abstract void Start();
        public abstract void Terminate();
        public void Stop() { ServiceIsRunning = false; }
        protected void WorkStarted() { IamWorking = true; }
        protected void WorkEnded() { IamWorking = true; }
        public bool ServiceIsRunning {
            get { return _ServiceIsRunning; }
            set {
                _ServiceIsRunning = value;
                if(!_ServiceIsRunning) {
                    Terminate();
                }
            }
        }
        public bool IamWorking {  get { return _IamWorking; } set { _IamWorking = value; } }

    }
    public class ConvertThread: ThreadWorkerBase {
        #region private members
        private string _WatchDir;
        private FileSystemWatcher _FileSystemWatcher;
        private WatcherQueue _Queue;
        private Timer _QueueResolver;
        private bool _IncludeSubdirectories = true;
        #endregion

        #region Constructors
        public ConvertThread(string watchDir, string outputDir) : base() {
            _WatchDir = watchDir;
            _Queue = new WatcherQueue();
        }
        #endregion

        #region Thread handling
        public override void Start() {
            EventLog.WriteEntry("ANTRAS.PdfConversion.Service", string.Format("ConvertThread started, observing {0}", _WatchDir), EventLogEntryType.Information, 0);
            AddAllExistingFilesToQueue();
            _FileSystemWatcher = new FileSystemWatcher();
            _FileSystemWatcher.Path = _WatchDir;
            _FileSystemWatcher.IncludeSubdirectories = _IncludeSubdirectories;
            _FileSystemWatcher.Created += new System.IO.FileSystemEventHandler(OnCreated);
            _FileSystemWatcher.EnableRaisingEvents = true;
            _QueueResolver = new Timer(500);
            _QueueResolver.Elapsed += new ElapsedEventHandler(QueueTimerTick);
            _QueueResolver.Start(); 
        }

        public override void Terminate() {
            //AntrasServiceContext.Log(" ! Termination Event Received (" + _TaskNo.ToString() + ") ");
            if(_QueueResolver != null) {
                _QueueResolver.Stop();
            }
            if(_FileSystemWatcher != null) {
                _FileSystemWatcher.EnableRaisingEvents = false;
            }
        }

        #endregion Thread handling

        #region on start
        private void AddAllExistingFilesToQueue() {
            DirectoryInfo di = new DirectoryInfo(_WatchDir);
            SearchOption searchOption = SearchOption.TopDirectoryOnly;
            if(_IncludeSubdirectories) {
                searchOption = SearchOption.AllDirectories;
            }
            foreach(FileInfo fi in di.GetFiles("*.xml", searchOption)) {
                _Queue.Enqueue(fi.FullName);
            }
            foreach(FileInfo fi in di.GetFiles("*.pdf", searchOption)) {
                _Queue.Enqueue(fi.FullName);
            }
        }
        #endregion

        #region FileWatcher events
        private void OnCreated(object sender, System.IO.FileSystemEventArgs e) {
            if(ServiceIsRunning) {
                string filePath = e.FullPath;
                _Queue.Enqueue(filePath);
            }
        }
        #endregion

        #region Timer Queue events
        private void QueueTimerTick(object sender, ElapsedEventArgs e) {
            _QueueResolver.Stop();
            WorkStarted();
            foreach(WatcherQueueEntry wqe in _Queue.EnQueued) {
                if(wqe.SecondsSinceAdded > 0.5) {
                    wqe.IncrementTryCount();
                    if(!IsFileLocked(wqe.Filename)) {
                        Request request = new Request(wqe.Filename);
                        request.Process();
                        _Queue.Dequeue(wqe);
                    }
                    if(wqe.TryCount > 200) {
                        _Queue.Dequeue(wqe);
                    }
                }
            }
            WorkEnded();
            if(ServiceIsRunning) {
                _QueueResolver.Start();
            }
        }
        #endregion

        #region helper methods
        private static bool IsFileLocked(string fileName) {
            bool ret = false;
            FileInfo file = new FileInfo(fileName);
            FileStream stream = null;
            try {
                stream = file.Open(FileMode.Open, FileAccess.ReadWrite, FileShare.None);
            } catch(IOException) {
                ret = true;
            } finally {
                if(stream != null) {
                    stream.Close();
                    stream.Dispose();
                }
            }
            return ret;
        }
        #endregion

        public string WatchDir { get { return _WatchDir; } }
    }

    public class WatcherQueue : ArrayList {
        public WatcherQueue() : base() { }
        public void Enqueue(string fileName) {
            if(!Contains(fileName)) {
                WatcherQueueEntry wqe = new WatcherQueueEntry(fileName);
                if(!wqe.IsDirectory) {
                    Add(wqe);
                }
            }
        }
        public void Dequeue(string fileName) { Dequeue(GetEntry(fileName)); }
        public void Dequeue(WatcherQueueEntry wqe) {
            if(wqe != null) {
                Remove(wqe);
            }
        }
        public override int Add(object value) { return base.Add(value); }
        public bool Contains(string fileName) {
            fileName = fileName.Trim();
            bool ret = false;
            foreach(WatcherQueueEntry wqe in this) {
                if(wqe.Filename.Equals(fileName)) {
                    ret = true;
                    break;
                }
            }
            return ret;
        }
        public WatcherQueueEntry GetEntry(string fileName) {
            fileName = fileName.Trim();
            WatcherQueueEntry ret = null;
            foreach(WatcherQueueEntry wqe in this) {
                if(wqe.Filename.Equals(fileName)) {
                    ret = wqe;
                    break;
                }
            }
            return ret;
        }
        public ArrayList EnQueued {
            get {
                ArrayList ret = new ArrayList();
                foreach(WatcherQueueEntry wqe in this) {
                    ret.Add(wqe);
                }
                return ret;
            }
        }
    }

    public class WatcherQueueEntry {
        private string _Filename;
        private int _TryCount = 0;
        private DateTime _EntryDateTime;
        private bool _IsDirectory = false;
        public WatcherQueueEntry(string fileName) {
            _Filename = fileName.Trim();
            _EntryDateTime = DateTime.Now;

            FileAttributes fa = File.GetAttributes(fileName);
            _IsDirectory = fa.Equals(FileAttributes.Directory);
        }

        public string Filename { get { return _Filename; } }
        public int IncrementTryCount() {
            _TryCount = _TryCount + 1;
            return _TryCount;
        }
        public double SecondsSinceAdded {
            get {
                TimeSpan diff = DateTime.Now - _EntryDateTime;
                return diff.TotalSeconds;
            }
        }
        public int TryCount { get { return _TryCount; } }
        public bool IsDirectory { get { return _IsDirectory; } }
    }

}
